@@ -32,6 +32,9 @@
 #include "sql_admin.h"
 #include "sql_statistics.h"
 #include "wsrep_mysqld.h"
+#ifdef WITH_WSREP
+#include "wsrep_trans_observer.h"
+#endif
 
 const LEX_CSTRING msg_status= {STRING_WITH_LEN("status")};
 const LEX_CSTRING msg_repair= { STRING_WITH_LEN("repair") };
@@ -45,7 +48,8 @@ const LEX_CSTRING msg_optimize= { STRING_WITH_LEN("optimize") };
 /* Prepare, run and cleanup for mysql_recreate_table() */
 
 static bool admin_recreate_table(THD *thd, TABLE_LIST *table_list,
-                                 Recreate_info *recreate_info)
+                                 Recreate_info *recreate_info,
+                                 bool table_copy)
 {
   bool result_code;
   DBUG_ENTER("admin_recreate_table");
@@ -66,7 +70,8 @@ static bool admin_recreate_table(THD *thd, TABLE_LIST *table_list,
   DEBUG_SYNC(thd, "ha_admin_try_alter");
   tmp_disable_binlog(thd); // binlogging is done by caller if wanted
   result_code= (thd->open_temporary_tables(table_list) ||
-                mysql_recreate_table(thd, table_list, recreate_info, false));
+                mysql_recreate_table(thd, table_list, recreate_info,
+                                     table_copy));
   reenable_binlog(thd);
   /*
     mysql_recreate_table() can push OK or ERROR.
@@ -283,6 +288,11 @@ static int prepare_for_repair(THD *thd, TABLE_LIST *table_list,
       goto end;
     }
   }
+  /*
+    We have now fixed the table. However the frm file is still of old format.
+    We cannot update the frm_file to FRM_VER_TRUE_VARCHAR as the new format
+    are not compatible with the data.
+  */
 
 end:
   thd->locked_tables_list.unlink_all_closed_tables(thd, NULL, 0);
@@ -445,6 +455,32 @@ dbug_err:
   return open_error;
 }
 
+#ifdef WITH_WSREP
+/** RAII class for temporarily disable wsrep_on in the connection. */
+class Disable_wsrep_on_guard
+{
+ public:
+  /**
+    @param thd     - pointer to the context of connection in which
+                     wsrep_on mode needs to be disabled.
+    @param disable - true if wsrep_on should be disabled
+  */
+  explicit Disable_wsrep_on_guard(THD *thd, bool disable)
+    : m_thd(thd), m_orig_wsrep_on(thd->variables.wsrep_on)
+  {
+    if (disable)
+      thd->variables.wsrep_on= false;
+  }
+
+  ~Disable_wsrep_on_guard()
+  {
+    m_thd->variables.wsrep_on= m_orig_wsrep_on;
+  }
+ private:
+  THD* m_thd;
+  bool m_orig_wsrep_on;
+};
+#endif /* WITH_WSREP */
 
 
 static void send_read_only_warning(THD *thd, const LEX_CSTRING *msg_status,
@@ -503,7 +539,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
                               const LEX_CSTRING *operator_name,
                               thr_lock_type lock_type,
                               bool org_open_for_modify,
-                              bool repair_table_use_frm,
+                              bool no_errors_from_open,
                               uint extra_open_options,
                               int (*prepare_func)(THD *, TABLE_LIST *,
                                                   HA_CHECK_OPT *),
@@ -517,8 +553,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
   List<Item> field_list;
   Protocol *protocol= thd->protocol;
   LEX *lex= thd->lex;
-  int result_code;
-  int compl_result_code;
   bool need_repair_or_alter= 0;
   wait_for_commit* suspended_wfc;
   bool is_table_modified= false;
@@ -526,6 +560,18 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
   DBUG_ENTER("mysql_admin_table");
   DBUG_PRINT("enter", ("extra_open_options: %u", extra_open_options));
 
+#ifdef WITH_WSREP
+  /*
+    CACHE INDEX and LOAD INDEX INTO CACHE statements are
+    local operations. Do not replicate them with Galera
+  */
+  const bool disable_wsrep_on= (WSREP(thd) &&
+    (lex->sql_command == SQLCOM_ASSIGN_TO_KEYCACHE ||
+     lex->sql_command == SQLCOM_PRELOAD_KEYS));
+
+  Disable_wsrep_on_guard wsrep_on_guard(thd, disable_wsrep_on);
+#endif /* WITH_WSREP */
+
   fill_check_table_metadata_fields(thd, &field_list);
 
   if (protocol->send_result_set_metadata(&field_list,
@@ -558,11 +604,14 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     uchar tabledef_version_buff[MY_UUID_SIZE];
     const char *db= table->db.str;
     bool fatal_error=0;
-    bool open_error= 0;
+    bool open_error= 0, recreate_used= 0;
+    bool require_data_conversion= 0, require_alter_table= 0;
     bool collect_eis=  FALSE;
     bool open_for_modify= org_open_for_modify;
     Recreate_info recreate_info;
+    int compl_result_code, result_code;
 
+    compl_result_code= result_code= HA_ADMIN_FAILED;
     storage_engine_name[0]= 0;                  // Marker that's not used
 
     DBUG_PRINT("admin", ("table: '%s'.'%s'", db, table->table_name.str));
@@ -582,7 +631,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
                                 ? MDL_SHARED_NO_READ_WRITE
                                 : lock_type >= TL_FIRST_WRITE
                                 ? MDL_SHARED_WRITE : MDL_SHARED_READ);
-
     if (thd->check_killed())
     {
       open_error= false;
@@ -595,7 +643,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     while (1)
     {
       open_error= open_only_one_table(thd, table,
-                                      repair_table_use_frm,
+                                      no_errors_from_open,
                                       (view_operator_func != NULL));
       thd->open_options&= ~extra_open_options;
 
@@ -778,7 +826,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
     if (lock_type == TL_WRITE && table->mdl_request.type > MDL_SHARED_WRITE)
     {
       if (table->table->s->tmp_table)
-        thd->close_unused_temporary_table_instances(tables);
+        thd->close_unused_temporary_table_instances(table);
       else
       {
         /* Store information about table for ddl log */
@@ -819,30 +867,41 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
       /* purecov: end */
     }
 
-    if (operator_func == &handler::ha_repair &&
-        !(check_opt->sql_flags & TT_USEFRM))
+    if (operator_func == &handler::ha_repair)
     {
       handler *file= table->table->file;
       int check_old_types=   file->check_old_types();
       int check_for_upgrade= file->ha_check_for_upgrade(check_opt);
+      require_data_conversion=
+        (check_old_types == HA_ADMIN_NEEDS_DATA_CONVERSION ||
+         check_for_upgrade == HA_ADMIN_NEEDS_DATA_CONVERSION);
+      require_alter_table=
+        (check_old_types == HA_ADMIN_NEEDS_ALTER ||
+         check_for_upgrade == HA_ADMIN_NEEDS_ALTER);
 
-      if (check_old_types == HA_ADMIN_NEEDS_ALTER ||
-          check_for_upgrade == HA_ADMIN_NEEDS_ALTER)
+      if (!(check_opt->sql_flags & (TT_USEFRM | TT_FORCE)))
       {
-        /* We use extra_open_options to be able to open crashed tables */
-        thd->open_options|= extra_open_options;
-        result_code= admin_recreate_table(thd, table, &recreate_info) ?
-                     HA_ADMIN_FAILED : HA_ADMIN_OK;
-        thd->open_options&= ~extra_open_options;
-        goto send_result;
-      }
-      if (check_old_types || check_for_upgrade)
-      {
-        /* If repair is not implemented for the engine, run ALTER TABLE */
-        need_repair_or_alter= 1;
+        if (require_data_conversion || require_alter_table)
+        {
+          /* We use extra_open_options to be able to open crashed tables */
+          thd->open_options|= extra_open_options;
+          result_code= (admin_recreate_table(thd, table, &recreate_info, 1) ?
+                        HA_ADMIN_FAILED : HA_ADMIN_OK);
+          recreate_used= 1;
+          thd->open_options&= ~extra_open_options;
+          goto send_result;
+        }
+        if (check_old_types || check_for_upgrade ||
+            !(table->table->file->ha_table_flags() & HA_CAN_REPAIR))
+        {
+          /*
+            If data upgrade is needed or repair is not implemented for the
+            engine, run ALTER TABLE FORCE
+          */
+          need_repair_or_alter= 1;
+        }
       }
     }
-
     result_code= compl_result_code= HA_ADMIN_OK;
 
     if (operator_func == &handler::ha_analyze)
@@ -879,6 +938,7 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
       DBUG_PRINT("admin", ("operator_func returned: %d", result_code));
     }
 
+    /* Note: compl_result_code can be different from result_code here */
     if (compl_result_code == HA_ADMIN_OK && collect_eis)
     {
       if (result_code == HA_ERR_TABLE_READONLY)
@@ -910,22 +970,46 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
       table->lock_type= TL_READ;
       DBUG_ASSERT(view_operator_func == NULL);
       open_error= open_only_one_table(thd, table,
-                                      repair_table_use_frm, FALSE);
+                                      no_errors_from_open, FALSE);
       thd->open_options&= ~extra_open_options;
 
       if (unlikely(!open_error))
       {
         TABLE *tab= table->table;
         Field **field_ptr= tab->field;
+        USED_MEM *memroot_block;
+
         if (!lex->column_list)
         {
+          /* Fields we have to read from the engine */
           bitmap_clear_all(tab->read_set);
+          /* Fields we want to have statistics for */
+          bitmap_clear_all(&tab->has_value_set);
+
           for (uint fields= 0; *field_ptr; field_ptr++, fields++)
           {
-            enum enum_field_types type= (*field_ptr)->type();
-            if (type < MYSQL_TYPE_MEDIUM_BLOB ||
+            Field *field= *field_ptr;
+            if (field->flags & LONG_UNIQUE_HASH_FIELD)
+            {
+              /*
+                No point in doing statistic for hash fields that should be
+                unique
+              */
+              continue;
+            }
+            /*
+              Note that type() always return MYSQL_TYPE_BLOB for
+              all blob types. Another function needs to be added
+              if we in the future want to distingush between blob
+              types here.
+            */
+            enum enum_field_types type= field->type();
+            if (type < MYSQL_TYPE_TINY_BLOB ||
                 type > MYSQL_TYPE_BLOB)
-              tab->field[fields]->register_field_in_read_map();
+            {
+              field->register_field_in_read_map();
+              bitmap_set_bit(&tab->has_value_set, field->field_index);
+            }
             else
               push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
                                   ER_NO_EIS_FOR_FIELD,
@@ -939,9 +1023,15 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
           LEX_STRING *column_name;
           List_iterator_fast<LEX_STRING> it(*lex->column_list);
 
+          /* Fields we have to read from the engine */
           bitmap_clear_all(tab->read_set);
+          /* Fields we want to have statistics for */
+          bitmap_clear_all(&tab->has_value_set);
+
           while ((column_name= it++))
           {
+            Field *field;
+            enum enum_field_types type;
             if (tab->s->fieldnames.type_names == 0 ||
                 (pos= find_type(&tab->s->fieldnames, column_name->str,
                                 column_name->length, 1)) <= 0)
@@ -950,10 +1040,15 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
               break;
             }
             pos--;
-            enum enum_field_types type= tab->field[pos]->type();
-            if (type < MYSQL_TYPE_MEDIUM_BLOB ||
-                type > MYSQL_TYPE_BLOB)
-              tab->field[pos]->register_field_in_read_map();
+            field= tab->field[pos];
+            type= field->type();
+            if (!(field->flags & LONG_UNIQUE_HASH_FIELD) &&
+                (type < MYSQL_TYPE_TINY_BLOB ||
+                 type > MYSQL_TYPE_BLOB))
+            {
+              field->register_field_in_read_map();
+              bitmap_set_bit(&tab->has_value_set, field->field_index);
+            }
             else
               push_warning_printf(thd, Sql_condition::WARN_LEVEL_WARN,
                                   ER_NO_EIS_FOR_FIELD,
@@ -969,7 +1064,6 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
           int pos;
           LEX_STRING *index_name;
           List_iterator_fast<LEX_STRING> it(*lex->index_list);
-
           tab->keys_in_use_for_query.clear_all();
           while ((index_name= it++))
           {
@@ -983,17 +1077,21 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
             tab->keys_in_use_for_query.set_bit(--pos);
           }
         }
+        /* Ensure that number of records are updated */
+        tab->file->info(HA_STATUS_VARIABLE);
+        memroot_block= get_last_memroot_block(thd->mem_root);
         if (!(compl_result_code=
-              alloc_statistics_for_table(thd, table->table)) &&
+              alloc_statistics_for_table(thd, tab,
+                                         &tab->has_value_set)) &&
             !(compl_result_code=
-              collect_statistics_for_table(thd, table->table)))
-          compl_result_code= update_statistics_for_table(thd, table->table);
+              collect_statistics_for_table(thd, tab)))
+          compl_result_code= update_statistics_for_table(thd, tab);
+        free_statistics_for_table(tab);
+        free_all_new_blocks(thd->mem_root, memroot_block);
       }
       else
         compl_result_code= HA_ADMIN_FAILED;
 
-      if (table->table)
-        free_statistics_for_table(thd, table->table);
       if (compl_result_code)
         result_code= HA_ADMIN_FAILED;
       else
@@ -1015,10 +1113,11 @@ static bool mysql_admin_table(THD* thd, TABLE_LIST* tables,
         repair was not implemented and we need to upgrade the table
         to a new version so we recreate the table with ALTER TABLE
       */
-      result_code= admin_recreate_table(thd, table, &recreate_info);
+      result_code= admin_recreate_table(thd, table, &recreate_info, 1);
+      recreate_used= 1;
     }
-send_result:
 
+send_result:
     lex->cleanup_after_one_table_open();
     thd->clear_error();  // these errors shouldn't get client
 
@@ -1059,6 +1158,34 @@ send_result:
       }
       thd->get_stmt_da()->clear_warning_info(thd->query_id);
     }
+
+    /*
+      Give a warning if REPAIR TABLE was used but table still needs an
+      ALTER TABLE. This can only happen for old type tables where REPAIR
+      was using FORCE to recover old data.
+    */
+    if (operator_func == &handler::ha_repair && ! recreate_used &&
+        (require_data_conversion || require_alter_table))
+    {
+      char buf[MYSQL_ERRMSG_SIZE];
+      if (result_code == HA_ADMIN_OK)
+      {
+        protocol->prepare_for_resend();
+        protocol->store(&table_name, system_charset_info);
+        protocol->store(operator_name, system_charset_info);
+	protocol->store(STRING_WITH_LEN("note"), system_charset_info);
+        protocol->store(STRING_WITH_LEN("Table data recovered"),system_charset_info);
+        if (protocol->write())
+          goto err;
+      }
+      my_snprintf(buf, sizeof(buf),
+                  ER_THD(thd, ER_TABLE_NEEDS_REBUILD),
+                  table_name.str);
+      if (send_check_errmsg(thd, table, operator_name, buf) < 0)
+        goto err;
+      result_code= HA_ADMIN_FAILED;
+    }
+
     protocol->prepare_for_resend();
     protocol->store(&table_name, system_charset_info);
     protocol->store(operator_name, system_charset_info);
@@ -1150,7 +1277,8 @@ send_result_message:
                  *save_next_global= table->next_global;
       table->next_local= table->next_global= 0;
 
-      result_code= admin_recreate_table(thd, table, &recreate_info);
+      result_code= admin_recreate_table(thd, table, &recreate_info, 0);
+      recreate_used= 1;
       trans_commit_stmt(thd);
       trans_commit(thd);
       close_thread_tables(thd);
@@ -1233,6 +1361,7 @@ send_result_message:
     }
 
     case HA_ADMIN_NEEDS_UPGRADE:
+    case HA_ADMIN_NEEDS_DATA_CONVERSION:
     case HA_ADMIN_NEEDS_ALTER:
     {
       char buf[MYSQL_ERRMSG_SIZE];
@@ -1241,7 +1370,7 @@ send_result_message:
           table->table->file->ha_table_flags() & HA_CAN_REPAIR ? "TABLE" : 0;
 
       protocol->store(&error_clex_str, system_charset_info);
-      if (what_to_upgrade)
+      if (what_to_upgrade && result_code == HA_ADMIN_NEEDS_UPGRADE)
         length= my_snprintf(buf, sizeof(buf),
                             ER_THD(thd, ER_TABLE_NEEDS_UPGRADE),
                             what_to_upgrade, table->table_name.str);
@@ -1280,13 +1409,8 @@ send_result_message:
 
     if (table->table && !table->view)
     {
-      /*
-        Don't skip flushing if we are collecting EITS statistics.
-      */
-      const bool skip_flush=
-        (operator_func == &handler::ha_analyze) && 
-        (table->table->file->ha_table_flags() & HA_ONLINE_ANALYZE) &&
-        !collect_eis;
+      /* Skip FLUSH TABLES if we are doing analyze */
+      const bool skip_flush= (operator_func == &handler::ha_analyze);
       if (table->table->s->tmp_table)
       {
         /*
@@ -1306,6 +1430,13 @@ send_result_message:
         table->table= 0;                        // For query cache
         query_cache_invalidate3(thd, table, 0);
       }
+      else if (collect_eis && skip_flush && compl_result_code == HA_ADMIN_OK)
+      {
+        TABLE_LIST *save_next_global= table->next_global;
+        table->next_global= 0;
+        read_statistics_for_tables(thd, table, true /* force_reload */);
+        table->next_global= save_next_global;
+      }
     }
     /* Error path, a admin command failed. */
     if (thd->transaction_rollback_request || fatal_error)
@@ -1532,7 +1663,8 @@ bool Sql_cmd_optimize_table::execute(THD *thd)
 
   WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, first_table);
   res= (specialflag & SPECIAL_NO_NEW_FUNC) ?
-    mysql_recreate_table(thd, first_table, &recreate_info, true) :
+    mysql_recreate_table(thd, first_table, &recreate_info,
+                         false) :
     mysql_admin_table(thd, first_table, &m_lex->check_opt,
                       &msg_optimize, TL_WRITE, 1, 0, 0, 0,
                       &handler::ha_optimize, 0, true);
@@ -1560,7 +1692,8 @@ bool Sql_cmd_repair_table::execute(THD *thd)
   WSREP_TO_ISOLATION_BEGIN_WRTCHK(NULL, NULL, first_table);
   res= mysql_admin_table(thd, first_table, &m_lex->check_opt, &msg_repair,
                          TL_WRITE, 1,
-                         MY_TEST(m_lex->check_opt.sql_flags & TT_USEFRM),
+                         MY_TEST(m_lex->check_opt.sql_flags &
+                                 (TT_USEFRM | TT_FORCE)),
                          HA_OPEN_FOR_REPAIR, &prepare_for_repair,
                          &handler::ha_repair, &view_repair, true);
 
