@@ -186,9 +186,7 @@ bool Alter_info::supports_lock(THD *thd, bool online,
   case HA_ALTER_INPLACE_EXCLUSIVE_LOCK:
     // If SHARED lock and no particular algorithm was requested, use COPY.
     if (requested_lock == Alter_info::ALTER_TABLE_LOCK_SHARED &&
-        algorithm(thd) == Alter_info::ALTER_TABLE_ALGORITHM_DEFAULT &&
-        thd->variables.alter_algorithm ==
-                Alter_info::ALTER_TABLE_ALGORITHM_DEFAULT)
+        algorithm(thd) == Alter_info::ALTER_TABLE_ALGORITHM_DEFAULT)
          return false;
 
     if (requested_lock == Alter_info::ALTER_TABLE_LOCK_SHARED ||
@@ -258,7 +256,7 @@ Alter_info::enum_alter_table_algorithm
 Alter_info::algorithm(const THD *thd) const
 {
   if (requested_algorithm == ALTER_TABLE_ALGORITHM_NONE)
-   return (Alter_info::enum_alter_table_algorithm) thd->variables.alter_algorithm;
+    return ALTER_TABLE_ALGORITHM_DEFAULT;
   return requested_algorithm;
 }
 
@@ -424,16 +422,15 @@ Alter_table_ctx::Alter_table_ctx(THD *thd, TABLE_LIST *table_list,
 
     if (lower_case_table_names == 1) // Convert new_name/new_alias to lower
     {
-      new_name.length= my_casedn_str(files_charset_info, (char*) new_name.str);
+      new_name= new_name_buff.copy_casedn(files_charset_info, new_name).
+                                to_lex_cstring();
       new_alias= new_name;
     }
     else if (lower_case_table_names == 2) // Convert new_name to lower case
     {
-      new_alias.str=    new_alias_buff;
-      new_alias.length= new_name.length;
-      strmov(new_alias_buff, new_name.str);
-      new_name.length= my_casedn_str(files_charset_info, (char*) new_name.str);
-
+      new_alias= new_name;
+      new_name= new_name_buff.copy_casedn(files_charset_info, new_name).
+                                to_lex_cstring();
     }
     else
       new_alias= new_name; // LCTN=0 => case sensitive + case preserving
@@ -461,7 +458,10 @@ Alter_table_ctx::Alter_table_ctx(THD *thd, TABLE_LIST *table_list,
                                tmp_file_prefix, current_pid, thd->thread_id);
   /* Safety fix for InnoDB */
   if (lower_case_table_names)
-    tmp_name.length= my_casedn_str(files_charset_info, tmp_name_buff);
+  {
+    // Ok to latin1, as the file name is in the form '#sql-alter-abc-def'
+    tmp_name.length= my_casedn_str_latin1(tmp_name_buff);
+  }
 
   if (table_list->table->s->tmp_table == NO_TMP_TABLE)
   {
@@ -654,19 +654,19 @@ bool Sql_cmd_alter_table::execute(THD *thd)
     }
 
     wsrep::key_array keys;
-    wsrep_append_fk_parent_table(thd, first_table, &keys);
-
-    WSREP_TO_ISOLATION_BEGIN_ALTER(lex->name.str ? select_lex->db.str
-                                   : first_table->db.str,
-                                   lex->name.str ? lex->name.str
-                                   : first_table->table_name.str,
-                                   first_table, &alter_info, &keys,
-                                   used_engine ? &create_info : nullptr)
+    if (!wsrep_append_fk_parent_table(thd, first_table, &keys))
     {
-      WSREP_WARN("ALTER TABLE isolation failure");
-      DBUG_RETURN(TRUE);
+      WSREP_TO_ISOLATION_BEGIN_ALTER(lex->name.str ? select_lex->db.str
+                                     : first_table->db.str,
+                                     lex->name.str ? lex->name.str
+                                     : first_table->table_name.str,
+                                     first_table, &alter_info, &keys,
+                                     used_engine ? &create_info : nullptr)
+      {
+        WSREP_WARN("ALTER TABLE isolation failure");
+        DBUG_RETURN(TRUE);
+      }
     }
-
     DEBUG_SYNC(thd, "wsrep_alter_table_after_toi");
   }
 #endif
